@@ -102,7 +102,10 @@ class Identifier:
         if target.group == TypeGroup.custom:
             if target == Types.external:
                 try:
-                    lambdas += [getattr(get_module_from_path(custom[0]), custom[1])]  # (module path, method name).
+                    lambda_ = getattr(get_module_from_path(custom[0]), custom[1])  # (module path, method name)
+                    if isinstance(lambda_, PickBase):
+                        lambda_ = lambda_.get_lambda(custom.pop(2) if len(custom) > 2 else None)
+                    lambdas.append(lambda_)
                 except IndexError:
                     raise ValueError(f"You must specify which method should be used in {custom[0]}")
                 except AttributeError:
@@ -239,6 +242,7 @@ class Identifier:
 
             if possible_types:  # sort by biggest score - biggest probability the column is of this type
                 field.possible_types = {k: v for k, v in sorted(possible_types.items(), key=lambda k: k[1], reverse=True)}
+            logger.debug(f"Possible type of the field '{field}': {possible_types}")
         return True
 
     def get_fitting_type(self, source_field_i, target_field, try_plaintext=False):
